package net.grandnature.android.oreilly.sensorvisualizer;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.View;

public class PlotView extends View {
	private Bitmap bmp;
	private Canvas canvas;
	private Paint plotPaint;
	private Paint baselinePaint;
	private int index;
	private int width;
	private int height;
	private int baseline;
	private float prev;
	public PlotView(Context context) {
		this(context, null);
	}
	public PlotView(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}
	public PlotView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		int plotColor = Color.WHITE, baselineColor = Color.WHITE;
		if(attrs != null) {
			TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.PlotView, 0, 0);
			plotColor = typedArray.getColor(R.styleable.PlotView_color_plot, plotColor);
			baselineColor = typedArray.getColor(R.styleable.PlotView_color_baseline, baselineColor);
		}
		canvas = new Canvas();
		index = 0;
		{
	        plotPaint = new Paint();
	        plotPaint.setAntiAlias(true);
	        plotPaint.setDither(true);
	        plotPaint.setColor(plotColor);
	        plotPaint.setStyle(Paint.Style.STROKE);
	        plotPaint.setStrokeWidth(1);
	        baselinePaint = new Paint();
	        baselinePaint.setAntiAlias(true);
	        baselinePaint.setDither(true);
	        baselinePaint.setColor(baselineColor);
	        baselinePaint.setStyle(Paint.Style.STROKE);
	        baselinePaint.setStrokeWidth(1);
		}
	}
    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        width = w;
        height = h;
        baseline = height/2;
        prev = baseline;
        bmp = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
        canvas.setBitmap(bmp);
        drawBaseline();
    }
	public void put(float val) {
		int newIndex = index + 1;
		float v = val > 0.0f ? (1 - val) * baseline : -val * baseline + baseline;
        canvas.drawLine(index, prev, newIndex, v, plotPaint);
        prev = v;
		index = newIndex;
        invalidate();
    }

	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
        if (bmp != null) {
			if(index >= width) {
				index = 0;
				bmp.eraseColor(Color.TRANSPARENT);
				drawBaseline();
			}
	        canvas.drawBitmap(bmp, 0, 0, null);
        }
	}
	private void drawBaseline() {
        canvas.drawLine(0, baseline, width, baseline, baselinePaint);
	}
}
