package net.grandnature.android.oreilly.whoop;

import net.grandnature.android.oreilly.whoop.sound.OnSoundDataReceivedListener;
import net.grandnature.android.oreilly.whoop.sound.SoundRecorderTask;
import android.app.Activity;
import android.os.Bundle;
import android.os.Vibrator;
import android.view.WindowManager;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebSettings.LayoutAlgorithm;

public class HtmlWhoop extends Activity implements OnSoundDataReceivedListener {
	private static final int FREQUENCY = 8000;
	private static Vibrator vibrator;
	private SoundRecorderTask task;
	private WebView webview;
	
	private ValueHolder values;
	private VolumeEvaluator evaluator;
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        webview = new WebView(this);
        WebSettings settings = webview.getSettings();
        settings.setJavaScriptEnabled(true);
        settings.setJavaScriptCanOpenWindowsAutomatically(true);
        settings.setLayoutAlgorithm(LayoutAlgorithm.NORMAL);
        webview.loadUrl("file:///android_asset/www/index.html");
        vibrator = (Vibrator) getSystemService(VIBRATOR_SERVICE);
        setContentView(webview);
        values = new ValueHolder(4);
        evaluator = new VolumeEvaluator();
    }

	@Override
	protected void onResume() {
		super.onResume();
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
        task = new SoundRecorderTask(FREQUENCY, this, this);
        task.execute();
	}
    
	@Override
	protected void onPause() {
        task.stop();
        task = null;
		getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		super.onPause();
	}

	@Override
	public void OnSoundDataReceived(short[] data) {
		float[] d = new float[data.length];
		for (int i = 0; i < data.length; i++) {
			d[i] = (float)data[i]/(float)Short.MAX_VALUE;
		}
		evaluator.normalizedData = d;
		runOnUiThread(evaluator);
	}
	private class VolumeEvaluator implements Runnable {
		private float[] normalizedData;
		@Override
		public void run() {
			if(normalizedData == null)
				return;
			float result = getPeak(normalizedData, 50);
			values.add(result);
			score();
		}
		private void score() {
			int score = values.getCountMoreThan(0.6f);
			webview.loadUrl("javascript:changeImage(" + score + ");"); 
			if(score == 4) {
				vibrator.vibrate(1000);
			}
		}
	}
	private float getPeak(float[] data, int num) {
		ValueHolder vh = new ValueHolder(num);
		int len = data.length;
		for (int i = 3; i < len-2; i++) {
			if(data[i] <= data[i-1])
				continue;
			if(data[i] <= data[i+1])
				continue;
			if(data[i-2] >= data[i] - 0.1f)
				continue;
			if(data[i+2] >= data[i] - 0.1f)
				continue;
			vh.add(data[i]);
		}
		return vh.getMedian();
	}
}