package net.grandnature.android.oreilly.whoop.sound;

import android.content.Context;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.AsyncTask;

public class SoundRecorderTask extends AsyncTask<Void, Void, Void> {
	private static final int audioEncoding = AudioFormat.ENCODING_PCM_16BIT;
	private static final int audioSource = MediaRecorder.AudioSource.MIC;
	private static final int channelConfiguration = AudioFormat.CHANNEL_CONFIGURATION_MONO;
	private int bufferSize, frequency;
	private volatile boolean on;
	private AudioRecord recorder;
	private OnSoundDataReceivedListener listener;

	public SoundRecorderTask(int frequency, Context context) {
		this(frequency, context, null);
	}
	public SoundRecorderTask(int frequency, Context context, OnSoundDataReceivedListener listener) {
		this.frequency = frequency;
		this.listener = listener;
	}
	public boolean isOn() { return on; }
	public void stop() { on = false; }
	@Override
	protected void onPreExecute() {
		super.onPreExecute();
		on = true;
	}
	@Override
	protected Void doInBackground(Void... params) {
		prepareRecorder();
		while (on) {
			short[] buf = new short[bufferSize];
			int readBytes = recorder.read(buf, 0, bufferSize);
			if(readBytes > 0) {
				if(listener!=null) {
					listener.OnSoundDataReceived(buf);
				}
			} else if (readBytes == AudioRecord.ERROR_INVALID_OPERATION || readBytes == AudioRecord.ERROR_BAD_VALUE) {

			}
		}
		stopRecorder();
		return null;
	}
	
	@Override
	protected void onPostExecute(Void result) {
		super.onPostExecute(result);
	}
	private void prepareRecorder() {
		bufferSize = AudioRecord.getMinBufferSize(frequency, channelConfiguration, audioEncoding);
		if(bufferSize < frequency)
			bufferSize = frequency;
		recorder = new AudioRecord(audioSource,
								   frequency, 
								   channelConfiguration, 
								   audioEncoding, 
								   bufferSize);
		recorder.startRecording();
	}
	private void stopRecorder() {
		recorder.stop();
		recorder.release();
	}
}
